#let size(mat) = {
  return str(mat.h) + "x" + str(mat.w)
}

#let mat(..args) = {
  let rows = args.pos()
  //rows = rows.map(r => if r == none {()} else {r})
  let h = rows.len()
  let w = if h == 0 {0} else {rows.first().len()}
  if rows.find(r => r.len() != w) != none {
    panic("All rows should have the same length")
  }
  return (
    rows: rows,
    w: w,
    h: h
  )
}

#let of-size(h, w) = {
  let rows = ((0,) * w,) * h
  return mat(..rows)
}

#let display(mat) = math.mat(..mat.rows)

#let add(mat1, mat2) = {
  if mat1.w != mat2.w or mat1.h != mat2.h {
    panic("Can't add matrices of size " + size(mat1) + " and " + size(mat2))
  }

  let mat3 = of-size(mat1.h, mat1.w)

  for y in range(mat1.h) {
    for x in range(mat1.w) {
      mat3.rows.at(y).at(x) = mat1.rows.at(y).at(x) + mat2.rows.at(y).at(x)
    }
  }

  return mat3
}

#let sub(mat1, mat2) = {
  if mat1.w != mat2.w or mat1.h != mat2.h {
    panic("Can't subtract matrices of size " + size(mat1) + " and " + size(mat2))
  }

  let mat3 = of-size(mat1.h, mat1.w)

  for y in range(mat1.h) {
    for x in range(mat1.w) {
      mat3.rows.at(y).at(x) = mat1.rows.at(y).at(x) - mat2.rows.at(y).at(x)
    }
  }

  return mat3
}

#let mul(mat1, mat2) = {
  if mat1.w != mat2.h {
    panic("Can't multiply matrices of size " + size(mat1) + " and " + size(mat2))
  }

  let mat3 = of-size(mat1.h, mat2.w)

  for y in range(mat1.h) {
    for x in range(mat2.w) {
      let sum = 0
      for i in range(mat1.w) {
        sum += mat1.rows.at(y).at(i) * mat2.rows.at(i).at(x)
      }
      mat3.rows.at(y).at(x) = sum
    }
  }

  return mat3
}

#let transpose(mat) = {
  let mat-t = of-size(mat.w, mat.h)

  for y in range(mat.h) {
    for x in range(mat.w) {
      mat-t.rows.at(x).at(y) = mat.rows.at(y).at(x)
    }
  }

  return mat-t
}

#let to-vec(mat) = {
  import "vec.typ" as _vec
  return _vec.vec(..mat.rows.map(r => r.first()))
}

#let mul-vec(mat1, vec) = {
  import "vec.typ" as _vec

  if mat1.w != vec.size {
    panic("Can't multiply matrix of size " + size(mat1) + " and vector of size " + _vec.size(vec))
  }

  let mat2 = _vec.to-mat(vec)
  let res = mul(mat1, mat2)

  return to-vec(res)
}